{ File   : SYBExt.dpr
  Author : Deniz Oezmen
  Created: 2005-04-25
  Changed: 2005-10-08

  Syberia 1/2 SYB extractor.
}
program SYBExt;

{$APPTYPE CONSOLE}

uses
  SysUtils;

type
  TSYBHeader = packed record
                 ID     : array[0..3] of Char; // 'VXBG'
                 DataOfs: Longword
               end;

  TSYBEntry  = packed record
                 Name  : string;
                 Offset,
                 Size  : Longword;
               end;

var
  SYBHeader: TSYBHeader;
  SYBFile  : file;

function OpenSYBFile(var SYBFile: file; SYBName: string): Boolean;
begin
  FileMode := fmShareDenyWrite;
  AssignFile(SYBFile, SYBName);

  {$I-}
  Reset(SYBFile, 1);
  {$I+}
  OpenSYBFile := (IOResult = 0)
end;

function ReadHeader: Boolean;
var
  BytesRead: Longword;
begin
  BlockRead(SYBFile, SYBHeader, SizeOf(TSYBHeader), BytesRead);
  Inc(SYBHeader.DataOfs, 8);
  ReadHeader := (BytesRead = SizeOf(TSYBHeader)) and (SYBHeader.ID = 'VXBG')
end;

procedure ExtractFiles;
var
  i, j,
  FileOfs : Longword;
  OutFile : file;
  c       : Char;
  Buffer  : array[0..32767] of Byte;
  FileList: array of TSYBEntry;
begin
  SetLength(FileList, 0);

  FileOfs := SYBHeader.DataOfs;

  while Longword(FilePos(SYBFile)) < SYBHeader.DataOfs do
  begin
    SetLength(FileList, High(FileList) + 2);

    with FileList[High(FileList)] do
    begin
      Name := '';
      repeat
        BlockRead(SYBFile, c, 1);
        if c <> #000 then
          Name := Name + c
      until c = #000;

      BlockRead(SYBFile, Size, 4);
      Offset := FileOfs;

      Inc(FileOfs, Size)
    end
  end;

  for i := 0 to High(FileList) do
  begin
    with FileList[i] do
    begin
      Writeln('Extracting ', Name, ' @ ', Offset, ' (', Size, ' bytes)');

      Seek(SYBFile, Offset);

      FileMode := fmOpenReadWrite;
      AssignFile(OutFile, Name);
      Rewrite(OutFile, 1);

      for j := 1 to Size div SizeOf(Buffer) do
      begin
        BlockRead(SYBFile, Buffer, SizeOf(Buffer));
        BlockWrite(OutFile, Buffer, SizeOf(Buffer))
      end;
      BlockRead(SYBFile, Buffer, Size mod SizeOf(Buffer));
      BlockWrite(OutFile, Buffer, Size mod SizeOf(Buffer));

      CloseFile(OutFile)
    end
  end
end;

procedure ParseFiles(FilePara: string);
var
  DirPos,
  i        : Byte;
  FileName,
  Dir,
  SYBName  : string;
  SearchRec: TSearchRec;
  PFilePara: PChar;
begin
  GetMem(PFilePara, Length(FilePara) + 1);
  StrPCopy(PFilePara, FilePara);

  // get the directory information from FilePara
  DirPos := 0;
  for i := 1 to Length(FilePara) do
    if FilePara[i] in [':', '\'] then
      DirPos := i;

  Dir := Copy(FilePara, 1, DirPos);

  FileMode := fmOpenRead;

  // search for all files matching FilePara
  if FindFirst(PFilePara, faAnyFile - faDirectory - faVolumeID,
    SearchRec) = 0 then
  begin
    repeat
      FileName := SearchRec.Name;
      SYBName := Dir + FileName;

      // if this file is accessible, analyze it
      if OpenSYBFile(SYBFile, SYBName) then
        if ReadHeader then
        begin
          Writeln('Extracting from ', SYBName, ' ...');
          ExtractFiles;
          Close(SYBFile)
        end
        else
          Writeln(SYBName, ' is not a valid SYB file.')
      else
        Writeln(SYBName, ' not accessible')
    until FindNext(SearchRec) <> 0
  end
  else
    Writeln(FilePara, ' not found')
end;

begin
  if ParamCount = 1 then
    ParseFiles(ParamStr(1))
  else
    Writeln('Incorrect number of parameters.')
end.

